/*
	EleBBS C Structures

	Converted from PASCAL (and some parts taken from the RA 2.50 CDK)
	to C.  Some comments are in C++ format but change them to the
	standard C format if you don't have a C++ compiler.

        These Structures are copyright (C) Michael Dillon.

	Converted by: Michael Dillon (GSValore)

        Tested with:    Visual C++ 4.0 Standard
                        DJGPP V2.952

        ** Use these structures at your own risk! I nor the authors
        ** of EleBBS and RemoteAccess will be responsible for any
        ** damage done by these structures.

        Structure Sizes:
        LIMITSrecord        :    80
        FILESHDRrecord      :   194
        MODEMrecord         :  1287
        TagFileRecord       :    50
        TelnetRecord        :   608
        LightbarRecord      :   375
        NewsArticleRecord   :   521
        NewsGroupStatRecord :   265
        NewsServerRecord    :   168
        NewsGroupStat       :   288
        ELECONFIGrecord     :  8445
        EleMessageRecord    :   287
        EleFilesRecord      :  1139
        USERONrecord        :   214

        Confirm these values when you compile your programs.

        EleBBS Copyright (C) Maarten Bekers.
        RemoteAccess Copyright (C) Bruce Morse.
        Visual C++ Copyright (C) Microsoft Corporation.
        DJGPP Copyright (C) D.J. Delorie
*/

#if !defined(ELEBBS_STRUCTURES__H)
#define ELEBBS_STRUCTURES__H

/* Requires the Real48toDouble header file */
#include "r48tdbl.h"

/* If the compiler is a Windows 32 or other ANSI C compiler, set the
   structure alignment to one.

   Be sure to add your compiler if it supports this option, so that
   it will pack the structures correctly.
*/
#if defined( _WIN32 ) || defined( DJGPP )
        #pragma pack ( 1 )
#endif

/* NOTICE:

EleBBS uses Turbo Pascal real variables which are 48-bit.
The variables that use the Turbo Pascal real form are marked as such.
You must convert these variable to a 64-bit double in order to properly
use them.
*/

/* Defined Message Kinds */
#define elemsgBoth     0
#define elemsgPrivate  1
#define elemsgPublic   2
#define elemsgROnly    3
#define elemsgNoReply  4

/* Reset Types */
#define Never 0
#define Week  1
#define Month 2
#define Year  3

/* Access Types */
#define nwsPostNever        0
#define nwsPostAlways       1
#define nwsPostUseSettings  2
#define nwsUseBoth          3

/* Type definitions required */
typedef signed char MSGTOIDXrecord[ 36 ];


/* Limits Structure
   Found in LIMITS.RA
*/
typedef struct {
	unsigned short int	Security;
	unsigned short int	Ltime;
	unsigned short int	L300;
	unsigned short int	L1200;
	unsigned short int	L2400;
	unsigned short int	L4800;
	unsigned short int	L7200;
	unsigned short int	L9600;
	unsigned short int	L12000;
	unsigned short int	L14400;
	unsigned short int	L16800;
	unsigned short int	L19200;
	unsigned short int	L38400;
	unsigned short int	Llocal;
	unsigned short int	RatioNum;
	unsigned short int	RatioK;
        TPReal                  PerMinCost;
	unsigned short int	L21600;
	unsigned short int	L24000;
	unsigned short int	L26400;
	unsigned short int	L28800;
	unsigned short int	L57600;
	unsigned short int	L64000;
        TPReal                  FlexiTime;
	unsigned short int	LsessionTime;
	unsigned short int	ResetAmt;
        unsigned char           ResetPeriod;     /* Values are defined through
                                                    Reset Types
                                                 */
	unsigned short int	ResetOffset;
	unsigned short int	L31200;
	unsigned short int	L33600;
	unsigned short int	L115200;
        unsigned char           FreeSpace[ 11 ];
} LIMITSrecord;

/* FILESHDR
   Found in FDBxxx.HD files
*/
typedef struct {
        signed char             Name[ 13 ];
	signed long int         Size;
	signed long int		CRC32;
        signed char             Uploader[ 36 ];
	signed long int		UploadDate;
	signed long int		FileDate;
	signed long int		LastDL;
	unsigned short int	TimesDL;
	unsigned char		Attrib;
        signed char             PassWord[ 16 ];
        signed char             KeyWord[ 5 ][ 16 ];
	unsigned short int	Cost;
	signed long int		LongDescPtr;
	signed long int		LfnPtr;
        signed char             FreeSpace[ 16 ];
} FILESHDRrecord;

/* NOTES on FILESHDR ***************************************************
Attrib:
Bit 0 = Deleted
Bit 1 = Unlisted
Bit 2 = Free (don't adjust ratio) - Does NOT affect "Cost"
Bit 3 = Not available (don't allow downloads)
Bit 4 = Locked (no kill)
Bit 5 = Missing/offline
Bit 6 = No time restrictions - always allow DL
***********************************************************************/

/* MODEM record
   Found in MODEM.RA
*/
typedef struct {
	unsigned char		ComPort;
	unsigned char		InitTries;
	unsigned short int	BufferSize;
	unsigned short int	ModemDelay;
	signed long int		MaxSpeed;
        unsigned char           SendBreak;      /* (BOOLEAN) */
        unsigned char           LockModem;      /* (BOOLEAN) */
        unsigned char           AnswerPhone;    /* (BOOLEAN) */
        unsigned char           OffHook;        /* (BOOLEAN) */
        signed char             InitStr[ 71 ];
        signed char             InitStr2[ 71 ];
        signed char             BusyStr[ 71 ];
        signed char             InitResp[ 41 ];
        signed char             BusyResp[ 41 ];
        signed char             Connect300[ 41 ];
        signed char             Connect1200[ 41 ];
        signed char             Connect2400[ 41 ];
        signed char             Connect4800[ 41 ];
        signed char             Connect7200[ 41 ];
        signed char             Connect9600[ 41 ];
        signed char             Connect12k[ 41 ];
        signed char             Connect14k[ 41 ];
        signed char             Connect16k[ 41 ];
        signed char             Connect19k[ 41 ];
        signed char             Connect38k[ 41 ];
        signed char             ConnectFax[ 41 ];
        signed char             RingStr[ 21 ];
        signed char             AnswerStr[ 21 ];
        signed char             ErrorFreeString[ 16 ];
        signed char             Connect21k[ 41 ];
        signed char             Connect24k[ 41 ];
        signed char             Connect26k[ 41 ];
        signed char             Connect28k[ 41 ];
        signed char             Connect57k[ 41 ];
        signed char             Connect64k[ 41 ];
        signed char             Connect31k[ 41 ];
        signed char             Connect33k[ 41 ];
        signed char             Connect115k[ 41 ];
        signed char             FreeSpace[ 59 ];
} MODEMrecord;

/* Found in RA v2.02 but never properly documented. */
/* For using LFN, use the RecordNum and Look it up */
typedef struct {
        signed char             Name[ 13 ];
        signed char             Password[ 16 ];
	unsigned char		Attrib;
	unsigned short int	AreaNum;
	unsigned short int	RecordNum;
	signed long int		Size;
        signed long int         FileDate;       /* Not Used                  */
	signed long int		Cost;
        unsigned char           CDROM;          /* (BOOLEAN)                 */
        unsigned char           FoundFirst;     /* (BOOLEAN) Internally used */
	unsigned short int	xFerTime;
} TagFileRecord;

/* Telnet layout record
   Found in TELNET.ELE
*/
typedef struct {
        signed long int         MaxSessions;    /* Max. Concurrent Sessions */
        signed long int         ServerPort;     /* Port listening on        */
        signed long int         StartNodeWith;  /* Starting Point for nodes */
        signed long int         Attrib; /* Bit 0 = Deny telnet download     */
                                        /* Bit 1 = Start EleBBS minimized   */
                                        /*         for telnet nodes         */

        signed char             ProgramPath[ 256 ];       /* Path to EleBBS.EXE     */
        signed char             NodeDirectories[ 256 ];   /* Directory TELSRV */
                                                          /* will switch to   */
        signed long int         FreeSpace[ 20 ];
} TelnetRecord;

/* LightBar Structure [for making lightbars]
   Found in <menu-name>.LBM files
*/
typedef struct {
        unsigned char           LightX;            /* X coord of the lightbar */
        unsigned char           LightY;            /* Y coord of the lightbar */
        signed char             LowItem[ 136 ];    /* Low-color item          */
        signed char             SelectItem[ 136 ]; /* Highlighted item        */
        unsigned char           Attrib;            /* Bit 0 = Enabled         */
        unsigned char           FreeSpace[ 100 ];
} LightbarRecord;


/*
   News Articles
   Found in NWSARTIC.ELE
*/
typedef struct {
        signed char             GroupName[ 101 ];
        signed long int         ArticleNr;      /* Article number in news    */
                                                /* group                     */
        signed long int         AreaNum;        /* Area Number in            */
                                                /* MESSAGES.RA               */
        signed long int         BodyLen;        /* Length of the text body   */
        signed long int         Attribute;      /* Bit 0 = Tossed into base? */
        signed long int         TimesSent;      /* Number of tries sending   */
                                                /* this msgs                 */
        signed long int         FreeSpace[ 100 ];
} NewsArticleRecord;

/*
   News Group Server Statistics
*/
typedef struct {
        signed char             GroupName[ 101 ];
	signed long int		LoMsgCount;
        signed long int         FreeSpace[ 40 ];
} NewsGroupStatRecord;

/* News Server Data
   Found in NWSGROUP.ELE
*/
typedef struct {
        signed long int         MaxSessions;    /* Max. concurrent sessions     */
        signed long int         ServerPort;     /* Port listening on            */
        signed long int         Attribute;      /* Bit 0 = Allow Sysop to logon */
        signed long int         FreeSpace[ 39 ];
} NewsServerRecord;

/*  News Group Statistics */
typedef struct {
        signed char             GroupName[ 101 ];    /* GroupName of these stats. */
        signed long int         NumRead;             /* Articles read in this     */
                                                     /* group                     */
        signed long int         NumWrites;           /* Articles written in this  */
                                                     /* group                     */
        signed long int         NumDeny;             /* Articles denied in this   */
                                                     /* group                     */
        signed char             LastAccessed[ 11 ];  /* Date this group was       */
                                                     /* last accessed             */
        signed long int         FreeSpace[ 41 ];
} NewsGroupStat;


/* Configuration Structure
   Found in CONFIG.ELE
*/
typedef struct {
        unsigned short int      VersionID;                 /* 0x05              */
        signed char             UtilityLogFileName[ 251 ]; /* Utilities-Servers */
                                                           /* Logfile name      */
	unsigned char		CapitalizeUserName;
                                /* (BOOLEAN) Capitalize User/Handle name? */
        signed char             AttachPassword[ 16 ];
                                /* Password for making attachments        */
        unsigned char           FreeSpace[ 8175 ];
} ELECONFIGrecord;

/* Message Base structure
   MESSAGES.ELE
*/
typedef struct {
	signed long int		AreaNum;
        signed char             GroupName[ 129 ];
        unsigned char           Attribute;        /* Bit 0 = Available for usenet */
        unsigned char           AccessSettings;   /* Values are defined under
                                                     the Access Types
                                                  */
        signed long int         AttachArea;       /* Area Number to put 
                                                     attachments in
                                                  */
        signed long int         FreeSpace[ 37 ];
} EleMessageRecord;

/* File base structure
   Found in FILES.ELE
*/
typedef struct {
	signed long int		AreaNum;
        signed char             ExportURL[ 251 ]; /* URL used by EleFILE */
                                                  /* HTMLIST             */
        signed char             ftpPath[ 251 ];
        signed char             ftpLoginName[ 36 ];
        signed char             ftpPassword[ 36 ];
	unsigned char		Attribute;
                                /* Bit 0 = Available for usenet */
        signed long int         FreeSpace[ 140 ];
} EleFilesRecord;

/* USERON Record
   Found in USERON.BBS
*/
typedef struct {
        MSGTOIDXrecord          Name;
        MSGTOIDXrecord          Handle;
	unsigned char		Line;
	unsigned short int      Baud;
        signed char             City[ 26 ];
	unsigned char		Status;
	unsigned char		Attribute;
        signed char             StatDesc[ 11 ];
        unsigned char           FreeSpace[ 90 ];
	unsigned short int	NoCalls;
	signed long int		NodeNumber;
        signed long int         LastUpdate; /* Unix timestamp (EleWEB only) */
} USERONrecord;

/** NOTES on USERONrecord ***********************
Status (Value):
  0 = Browsing (in a menu)
  1 = Uploading/downloading
  2 = Reading/posting messages
  3 = In a door/external utility
  4 = Chatting with sysop
  5 = Answering questionnaire
  6 = RTC (Real Time Chat?)
  7 = New user logon
255 = User-defined - display StatDesc

Attribute:
Bit 0 = Hidden
Bit 1 = Wants chat
Bit 2 = Reserved for RANETMGR
Bit 3 = Do not disturb flag
Bit 6 = Ready (0=busy)
**************************************************************/

#endif

/* -- END OF FILE -- */
