/*********************************************************************/
/* Real48 <-> Double Test Program                                    */
/*                                                                   */
/* The purpose of this file is to give an example of how to read and */
/* write Real48 variables to and from a file.                        */
/*                                                                   */
/* You may use this program freely provided you do not change the    */
/* Copyright or other vital information.                             */
/*                                                                   */
/* This program was tested and compiled with:                        */
/*                                                                   */
/* DJGPP 2.952                                                       */
/* Visual C++ 4.0 Standard                                           */
/*                                                                   */
/* Use these files at your own risk.  There is no warranty for       */
/* them and they are provided AS IS.  I nor the author of EleBBS     */
/* will be held responsible for any damages these programs may       */
/* cause!                                                            */
/*                                                                   */
/* Copyright (C) 2000 Michael Dillon.  All Rights Reserved.          */
/* Created on May 18, 2000.                                          */
/*********************************************************************/

/* Include the required headers                  */
#include <stdio.h>
#include <string.h>
#include <math.h>

/* Include the Real48 to Double function header. */
#include "r48tdbl.h"


/* This will set the structure alignment to 1 provided it is   */
/* compiled under a ANSI compliant compiler.  If your compiler */
/* supports this option and it is not listed, then modify the  */
/* following lines to assure your compiler sets the alignment  */
/* correctly.                                                  */
#if defined( _WIN32 ) || defined( DJGPP )
        #pragma pack( 1 )
#endif

/* Constants                                            */
/* __FileNameIn refers to the file that contains one    */
/*              Real48 value to be read.                */
/*                                                      */
/* __FileNameOut refers to file that will contain a     */
/*               Real48 value to be written.            */
const unsigned char     *__FileNameIn  = "Real48.in";
const unsigned char     *__FileNameOut = "Real48.out";

int main( void )
{
        /* Variables used by this program  */
        double final;
        TPReal TPR;
        FILE *file; 

        /* Zero the entire contents of TPR */
        memset(&TPR, 0, sizeof(TPR));
       
        /* Open a __FileNameIn for binary read */
        if ( ( file = fopen( __FileNameIn, "rb" ) ) == NULL )
        {
                /* If unable to open, output the error */
                fprintf( stderr, "Unable to open %s for reading!\n", __FileNameIn );
        }
        else
        {
                /* Read in the value of the Real48 variable contained */
                /* in the file to the TPReal structure.               */
                if ( fread( &TPR, sizeof( TPR ), 1, file) != 1 )
                {
                        /* If fread() was unable to read the file, then */
                        /* output to the user.                          */
                        fprintf( stderr, "Unable to read Real48 value from %s!\n", __FileNameIn );
                }
                else
                {
                        /* Use the TPReal structure that was read using fread()   */
                        /* and convert it to a double giving the value to "final" */
                        final = Real48ToDouble( TPR );

                        /* Output the converted result                            */
                        printf( "Conversion gave: %0.20f\n", final );
                }

                /* Close the file stream */
                fclose(file);
        }
                                        
        /* Open __FileNameOut, this file will hold the TPReal variable that */
        /* will be converted from a double.                                 */
        if ( ( file = fopen( __FileNameOut, "wb" ) ) == NULL )
        {       
                /* If it was unable to open the file, warn the user */
                fprintf( stderr, "Unable to open %s for writing!\n", __FileNameOut );
        }
        else
        {
                /* Clear out the contents of TPR */
                memset( &TPR, 0, sizeof( TPR ) );

                /* Convert the value "final" which was carried over */
                /* to a Real48 value.                               */
                TPR = DoubleToReal48( final );

                /* Write the Real48 converted variable to the new   */
                /* file.                                            */
                if ( fwrite( &TPR, sizeof( TPR ), 1, file) != 1 )
                {
                        /* If unable to write, then warn the user   */
                        fprintf( stderr, "Unable to write to %s!\n", __FileNameOut );
                }

                /* Close the file stream */
                fclose(file);
        }

        /* Output the sizes of TPReal and DoubleStruct */
        /* TPReal must be 6 and DoubleStruct must be 8 */
        printf( "Size of TPReal: %05d\n", sizeof( TPReal ) );
        printf( "Size of DoubleStruct: %05d\n", sizeof( DoubleStruct ) );

        /* Exit */
        return ( 0 );
}

